package org.bitbucket.bradleysmithllc.etlunit.feature;

import com.fasterxml.jackson.databind.JsonNode;
import com.github.fge.jackson.JsonLoader;
import com.github.fge.jsonschema.exceptions.ProcessingException;
import com.github.fge.jsonschema.main.JsonSchema;
import com.github.fge.jsonschema.main.JsonSchemaFactory;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

public class ResourceFeatureAnnotation implements FeatureAnnotation
{
	private final String name;
	private final String description;
	private final ResourceFeatureMetaInfo resourceFeatureMetaInfo;
	private final propertyType pType;

	public ResourceFeatureAnnotation(ResourceFeatureMetaInfo resourceFeatureMetaInfo, String key, String description, propertyType prop)
	{
		name = key;
		this.description = description;
		this.resourceFeatureMetaInfo = resourceFeatureMetaInfo;
		pType = prop;
	}

	@Override
	public String getName()
	{
		return name;
	}

	@Override
	public String getDescription()
	{
		return description;
	}

	@Override
	public List<JsonSchema> getValidator()
	{
		JsonNode validatorNode = resourceFeatureMetaInfo.loadValidatorNode(name);

		if (validatorNode != null)
		{
			return Arrays.asList(resourceFeatureMetaInfo.loadValidator(validatorNode, ""));
		}

		return null;
	}

	@Override
	public propertyType getPropertyType()
	{
		return pType;
	}
}