package org.bitbucket.bradleysmithllc.etlunit.metadata.impl;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.google.gson.stream.JsonWriter;
import org.bitbucket.bradleysmithllc.etlunit.RuntimeSupport;
import org.bitbucket.bradleysmithllc.etlunit.metadata.MetaData;
import org.bitbucket.bradleysmithllc.etlunit.metadata.MetaDataContext;

import javax.inject.Inject;
import java.io.IOException;
import java.util.*;

public class MetaDataImpl implements MetaData
{
	private RuntimeSupport runtimeSupport;

	private final List<MetaDataContext> contexts = new ArrayList<MetaDataContext>();
	private final Map<String, MetaDataContext> contextMap = new HashMap<String, MetaDataContext>();

	@Inject
	public void receiveRuntimeSupport(RuntimeSupport support)
	{
		runtimeSupport = support;
	}

	@Override
	public MetaDataContext getOrCreateContext(String name) {
		if (contextMap.containsKey(name))
		{
			return contextMap.get(name);
		}

		MetaDataContextImpl mimpl = new MetaDataContextImpl(name);
		mimpl.receiveRuntimeSupport(runtimeSupport);

		contexts.add(mimpl);
		contextMap.put(name, mimpl);

		return mimpl;
	}

	@Override
	public Map<String, MetaDataContext> getContextMap() {
		return Collections.unmodifiableMap(contextMap);
	}

	@Override
	public List<MetaDataContext> getContexts() {
		return Collections.unmodifiableList(contexts);
	}

	@Override
	public void toJson(JsonWriter writer) throws IOException {
		writer.name("meta-data");

		writer.beginArray();

		for (MetaDataContext ctx : contexts)
		{
			writer.beginObject();
			ctx.toJson(writer);
			writer.endObject();
		}

		writer.endArray();
	}

	@Override
	public void fromJson(JsonNode node) {
		contextMap.clear();
		contexts.clear();

		ArrayNode anode = (ArrayNode) node.get("meta-data");

		Iterator<JsonNode> it = anode.elements();

		while (it.hasNext())
		{
			MetaDataContextImpl mimpl = new MetaDataContextImpl();
			mimpl.receiveRuntimeSupport(runtimeSupport);

			mimpl.fromJson(it.next());

			contexts.add(mimpl);
			contextMap.put(mimpl.getContextName(), mimpl);
		}
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;

		MetaDataImpl metaData = (MetaDataImpl) o;

		if (!contexts.equals(metaData.contexts)) return false;

		return true;
	}

	@Override
	public int hashCode() {
		return contexts.hashCode();
	}
}