package org.bitbucket.bradleysmithllc.etlunit.metadata.impl;

import com.fasterxml.jackson.databind.JsonNode;
import com.github.fge.jackson.JsonLoader;
import com.google.gson.stream.JsonWriter;
import org.apache.commons.io.FileUtils;
import org.bitbucket.bradleysmithllc.etlunit.RuntimeSupport;
import org.bitbucket.bradleysmithllc.etlunit.metadata.MetaData;
import org.bitbucket.bradleysmithllc.etlunit.metadata.MetaDataManager;

import javax.inject.Inject;
import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;

public class MetaDataManagerImpl implements MetaDataManager
{
	private RuntimeSupport runtimeSupport;
	private MetaDataImpl metaData = new MetaDataImpl();

	@Inject
	public void receiveRuntimeSupport(RuntimeSupport support)
	{
		runtimeSupport = support;
		metaData.receiveRuntimeSupport(runtimeSupport);
	}

	@Override
	public JsonNode toJson(MetaData data) {
		StringWriter writer = new StringWriter();
		JsonWriter jwriter = new JsonWriter(writer);

		try {
			jwriter.beginObject();
			data.toJson(jwriter);
			jwriter.endObject();

			jwriter.close();
			return JsonLoader.fromString(writer.toString());
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	@Override
	public MetaData fromJson(JsonNode node) {
		MetaDataImpl mdimpl = new MetaDataImpl();
		mdimpl.receiveRuntimeSupport(runtimeSupport);
		mdimpl.fromJson(node);

		return mdimpl;
	}

	@Override
	public MetaData getMetaData() {
		return metaData;
	}

	@Override
	public void dispose() {
		// write out meta data json catalog
		File tgt = runtimeSupport.createGeneratedSourceFile("metadata", "meta-data.json");

		JsonNode jnode = toJson(metaData);

		try {
			FileUtils.write(tgt, jnode.toString());
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
}
